@@ -32,9 +32,25 @@ var _ = Describe("Node Operations", func() {
 	It("sets correct OVN external IDs", func() {
 		app.Action = func(ctx *cli.Context) error {
 			const (
-				nodeName string = "1.2.5.6"
+				nodeIP   string = "1.2.5.6"
+				nodeName string = "cannot.be.resolv.ed"
 				interval int    = 100000
+				ofintval int    = 180
 			)
+			node := kapi.Node{
+				Status: kapi.NodeStatus{
+					Addresses: []kapi.NodeAddress{
+						{
+							Type:    kapi.NodeHostName,
+							Address: nodeName,
+						},
+						{
+							Type:    kapi.NodeExternalIP,
+							Address: nodeIP,
+						},
+					},
+				},
+			}
 
 			fexec := ovntest.NewFakeExec()
 			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
@@ -42,8 +58,9 @@ var _ = Describe("Node Operations", func() {
 					"external_ids:ovn-encap-type=geneve "+
 					"external_ids:ovn-encap-ip=%s "+
 					"external_ids:ovn-remote-probe-interval=%d "+
+					"external_ids:ovn-openflow-probe-interval=%d "+
 					"external_ids:hostname=\"%s\"",
-					nodeName, interval, nodeName),
+					nodeIP, interval, ofintval, nodeName),
 			})
 
 			err := util.SetExec(fexec)
@@ -52,122 +69,83 @@ var _ = Describe("Node Operations", func() {
 			_, err = config.InitConfig(ctx, fexec, nil)
 			Expect(err).NotTo(HaveOccurred())
 
-			err = setupOVNNode(nodeName)
+			err = setupOVNNode(&node)
 			Expect(err).NotTo(HaveOccurred())
 
-			Expect(fexec.CalledMatchesExpected()).To(BeTrue())
+			Expect(fexec.CalledMatchesExpected()).To(BeTrue(), fexec.ErrorDesc)
 			return nil
 		}
 
 		err := app.Run([]string{app.Name})
 		Expect(err).NotTo(HaveOccurred())
 	})
-	It("test validateOVNConfigEndpoint()", func() {
-
-		type testcase struct {
-			name           string
-			subsets        []kapi.EndpointSubset
-			expectedResult bool
-		}
-
-		testcases := []testcase{
-			{
-				name: "valid endpoint",
-				subsets: []kapi.EndpointSubset{
-					{
-						Addresses: []kapi.EndpointAddress{
-							{IP: "10.1.2.3"},
+	It("sets non-default OVN encap port", func() {
+		app.Action = func(ctx *cli.Context) error {
+			const (
+				nodeIP      string = "1.2.5.6"
+				nodeName    string = "cannot.be.resolv.ed"
+				encapPort   uint   = 666
+				interval    int    = 100000
+				ofintval    int    = 180
+				chassisUUID string = "1a3dfc82-2749-4931-9190-c30e7c0ecea3"
+				encapUUID   string = "e4437094-0094-4223-9f14-995d98d5fff8"
+			)
+			node := kapi.Node{
+				Status: kapi.NodeStatus{
+					Addresses: []kapi.NodeAddress{
+						{
+							Type:    kapi.NodeHostName,
+							Address: nodeName,
 						},
-						Ports: []kapi.EndpointPort{
-							{
-								Name: "north",
-								Port: 1234,
-							},
-							{
-								Name: "south",
-								Port: 4321,
-							},
+						{
+							Type:    kapi.NodeExternalIP,
+							Address: nodeIP,
 						},
 					},
 				},
-				expectedResult: true,
-			},
-			{
-				name: "valid endpoint, multiple IPs",
-				subsets: []kapi.EndpointSubset{
-					{
-						Addresses: []kapi.EndpointAddress{
-							{IP: "10.1.2.3"}, {IP: "11.1.2.3"},
-						},
-						Ports: []kapi.EndpointPort{
-							{
-								Name: "north",
-								Port: 1234,
-							},
-							{
-								Name: "south",
-								Port: 4321,
-							},
-						},
-					},
-				},
-				expectedResult: true,
-			},
-			{
-				name: "invalid endpoint two few ports",
-				subsets: []kapi.EndpointSubset{
-					{
-						Addresses: []kapi.EndpointAddress{
-							{IP: "10.1.2.3"},
-						},
-						Ports: []kapi.EndpointPort{
-							{
-								Name: "north",
-								Port: 1234,
-							},
-						},
-					},
-				},
-				expectedResult: false,
-			},
-			{
-				name: "invalid endpoint too many ports",
-				subsets: []kapi.EndpointSubset{
-					{
-						Addresses: []kapi.EndpointAddress{
-							{IP: "10.1.2.3"},
-						},
-						Ports: []kapi.EndpointPort{
-							{
-								Name: "north",
-								Port: 1234,
-							},
-							{
-								Name: "south",
-								Port: 4321,
-							},
-							{
-								Name: "east",
-								Port: 7654,
-							},
-						},
-					},
-				},
-				expectedResult: false,
-			},
-			{
-				name:           "invalid endpoint no subsets",
-				subsets:        []kapi.EndpointSubset{},
-				expectedResult: false,
-			},
-		}
-
-		for _, tc := range testcases {
-			test := kapi.Endpoints{
-				Subsets: tc.subsets,
 			}
-			Expect(validateOVNConfigEndpoint(&test)).To(Equal(tc.expectedResult), " test case \"%s\" returned %t instead of %t", tc.name, !tc.expectedResult, tc.expectedResult)
+
+			fexec := ovntest.NewFakeExec()
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
+				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . "+
+					"external_ids:ovn-encap-type=geneve "+
+					"external_ids:ovn-encap-ip=%s "+
+					"external_ids:ovn-remote-probe-interval=%d "+
+					"external_ids:ovn-openflow-probe-interval=%d "+
+					"external_ids:hostname=\"%s\"",
+					nodeIP, interval, ofintval, nodeName),
+			})
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
+				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 " +
+					"--if-exists get Open_vSwitch . external_ids:system-id"),
+				Output: chassisUUID,
+			})
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
+				Cmd: fmt.Sprintf("ovn-sbctl --timeout=15 --data=bare --no-heading --columns=_uuid find "+
+					"Encap chassis_name=%s", chassisUUID),
+				Output: encapUUID,
+			})
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
+				Cmd: fmt.Sprintf("ovn-sbctl --timeout=15 set encap "+
+					"%s options:dst_port=%d", encapUUID, encapPort),
+			})
+
+			err := util.SetExec(fexec)
+			Expect(err).NotTo(HaveOccurred())
+
+			_, err = config.InitConfig(ctx, fexec, nil)
+			Expect(err).NotTo(HaveOccurred())
+			config.Default.EncapPort = encapPort
+
+			err = setupOVNNode(&node)
+			Expect(err).NotTo(HaveOccurred())
+
+			Expect(fexec.CalledMatchesExpected()).To(BeTrue(), fexec.ErrorDesc)
+			return nil
 		}
+
+		err := app.Run([]string{app.Name})
+		Expect(err).NotTo(HaveOccurred())
 	})
 	It("test watchConfigEndpoints single IP", func() {
 		app.Action = func(ctx *cli.Context) error {
@@ -181,14 +159,14 @@ var _ = Describe("Node Operations", func() {
 			fexec := ovntest.NewFakeExec()
 			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . "+
-					"external_ids:ovn-nb=\"tcp:%s:%d\"",
-					masterAddress, nbPort),
+					"external_ids:ovn-nb=\"tcp:%s\"",
+					util.JoinHostPortInt32(masterAddress, nbPort)),
 			})
 
 			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . "+
-					"external_ids:ovn-remote=\"tcp:%s:%d\"",
-					masterAddress, sbPort),
+					"external_ids:ovn-remote=\"tcp:%s\"",
+					util.JoinHostPortInt32(masterAddress, sbPort)),
 			})
 
 			err := util.SetExec(fexec)
@@ -222,7 +200,7 @@ var _ = Describe("Node Operations", func() {
 			stopChan := make(chan struct{})
 			f, err := factory.NewWatchFactory(fakeClient, stopChan)
 			Expect(err).NotTo(HaveOccurred())
-			defer f.Shutdown()
+			defer close(stopChan)
 
 			cluster := NewClusterController(fakeClient, f)
 			Expect(cluster).NotTo(BeNil())
@@ -230,16 +208,16 @@ var _ = Describe("Node Operations", func() {
 			Expect(config.OvnNorth.Address).To(Equal("tcp:1.1.1.1:6641"), "config.OvnNorth.Address does not equal cli arg")
 			Expect(config.OvnSouth.Address).To(Equal("tcp:1.1.1.1:6642"), "config.OvnSouth.Address does not equal cli arg")
 
-			err = cluster.watchConfigEndpoints()
+			err = cluster.watchConfigEndpoints(make(chan bool, 1))
 			Expect(err).NotTo(HaveOccurred())
 
 			// Kubernetes endpoints should eventually propogate to OvnNorth/OvnSouth
 			Eventually(func() string {
 				return config.OvnNorth.Address
-			}).Should(Equal(fmt.Sprintf("tcp:%s:%d", masterAddress, nbPort)), "Northbound DB Port did not get set by watchConfigEndpoints")
+			}).Should(Equal(fmt.Sprintf("tcp:%s", util.JoinHostPortInt32(masterAddress, nbPort))), "Northbound DB Port did not get set by watchConfigEndpoints")
 			Eventually(func() string {
 				return config.OvnSouth.Address
-			}).Should(Equal(fmt.Sprintf("tcp:%s:%d", masterAddress, sbPort)), "Southbound DBPort did not get set by watchConfigEndpoints")
+			}).Should(Equal(fmt.Sprintf("tcp:%s", util.JoinHostPortInt32(masterAddress, sbPort))), "Southbound DBPort did not get set by watchConfigEndpoints")
 
 			return nil
 		}
@@ -247,6 +225,7 @@ var _ = Describe("Node Operations", func() {
 		Expect(err).NotTo(HaveOccurred())
 
 	})
+
 	It("test watchConfigEndpoints multiple IPs", func() {
 		app.Action = func(ctx *cli.Context) error {
 
@@ -260,14 +239,16 @@ var _ = Describe("Node Operations", func() {
 			fexec := ovntest.NewFakeExec()
 			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . "+
-					"external_ids:ovn-nb=\"tcp:%s:%d,tcp:%s:%d\"",
-					masterAddress1, nbPort, masterAddress2, nbPort),
+					"external_ids:ovn-nb=\"tcp:%s,tcp:%s\"",
+					util.JoinHostPortInt32(masterAddress1, nbPort),
+					util.JoinHostPortInt32(masterAddress2, nbPort)),
 			})
 
 			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
 				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . "+
-					"external_ids:ovn-remote=\"tcp:%s:%d,tcp:%s:%d\"",
-					masterAddress1, sbPort, masterAddress2, sbPort),
+					"external_ids:ovn-remote=\"tcp:%s,tcp:%s\"",
+					util.JoinHostPortInt32(masterAddress1, sbPort),
+					util.JoinHostPortInt32(masterAddress2, sbPort)),
 			})
 
 			err := util.SetExec(fexec)
@@ -301,7 +282,7 @@ var _ = Describe("Node Operations", func() {
 			stopChan := make(chan struct{})
 			f, err := factory.NewWatchFactory(fakeClient, stopChan)
 			Expect(err).NotTo(HaveOccurred())
-			defer f.Shutdown()
+			defer close(stopChan)
 
 			cluster := NewClusterController(fakeClient, f)
 			Expect(cluster).NotTo(BeNil())
@@ -309,16 +290,16 @@ var _ = Describe("Node Operations", func() {
 			Expect(config.OvnNorth.Address).To(Equal("tcp:1.1.1.1:6641"), "config.OvnNorth.Address does not equal cli arg")
 			Expect(config.OvnSouth.Address).To(Equal("tcp:1.1.1.1:6642"), "config.OvnSouth.Address does not equal cli arg")
 
-			err = cluster.watchConfigEndpoints()
+			err = cluster.watchConfigEndpoints(make(chan bool, 1))
 			Expect(err).NotTo(HaveOccurred())
 
 			// Kubernetes endpoints should eventually propogate to OvnNorth/OvnSouth
 			Eventually(func() string {
 				return config.OvnNorth.Address
-			}).Should(Equal(fmt.Sprintf("tcp:%s:%d,tcp:%s:%d", masterAddress1, nbPort, masterAddress2, nbPort)), "Northbound DB Port did not get set by watchConfigEndpoints")
+			}).Should(Equal(fmt.Sprintf("tcp:%s,tcp:%s", util.JoinHostPortInt32(masterAddress1, nbPort), util.JoinHostPortInt32(masterAddress2, nbPort))), "Northbound DB Port did not get set by watchConfigEndpoints")
 			Eventually(func() string {
 				return config.OvnSouth.Address
-			}).Should(Equal(fmt.Sprintf("tcp:%s:%d,tcp:%s:%d", masterAddress1, sbPort, masterAddress2, sbPort)), "Southbound DBPort did not get set by watchConfigEndpoints")
+			}).Should(Equal(fmt.Sprintf("tcp:%s,tcp:%s", util.JoinHostPortInt32(masterAddress1, sbPort), util.JoinHostPortInt32(masterAddress2, sbPort))), "Southbound DBPort did not get set by watchConfigEndpoints")
 
 			return nil
 		}
@@ -326,4 +307,88 @@ var _ = Describe("Node Operations", func() {
 		Expect(err).NotTo(HaveOccurred())
 
 	})
+
+	It("test watchConfigEndpoints to learn nb and sb addresses", func() {
+		app.Action = func(ctx *cli.Context) error {
+
+			const (
+				masterAddress1 string = "10.10.2.3"
+				masterAddress2 string = "11.10.2.3"
+				nbPort         int32  = 5678
+				sbPort         int32  = 8765
+			)
+
+			fexec := ovntest.NewFakeExec()
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
+				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . "+
+					"external_ids:ovn-nb=\"tcp:%s,tcp:%s\"",
+					util.JoinHostPortInt32(masterAddress1, nbPort),
+					util.JoinHostPortInt32(masterAddress2, nbPort)),
+			})
+
+			fexec.AddFakeCmd(&ovntest.ExpectedCmd{
+				Cmd: fmt.Sprintf("ovs-vsctl --timeout=15 set Open_vSwitch . "+
+					"external_ids:ovn-remote=\"tcp:%s,tcp:%s\"",
+					util.JoinHostPortInt32(masterAddress1, sbPort),
+					util.JoinHostPortInt32(masterAddress2, sbPort)),
+			})
+
+			err := util.SetExec(fexec)
+			Expect(err).NotTo(HaveOccurred())
+
+			fakeClient := fake.NewSimpleClientset(&kapi.EndpointsList{
+				Items: []kapi.Endpoints{{
+					ObjectMeta: metav1.ObjectMeta{Namespace: "ovn-kubernetes", Name: "ovnkube-db"},
+					Subsets: []kapi.EndpointSubset{
+						{
+							Addresses: []kapi.EndpointAddress{
+								{IP: masterAddress1}, {IP: masterAddress2},
+							},
+							Ports: []kapi.EndpointPort{
+								{
+									Name: "north",
+									Port: nbPort,
+								},
+								{
+									Name: "south",
+									Port: sbPort,
+								},
+							},
+						},
+					},
+				}},
+			})
+			_, err = config.InitConfig(ctx, fexec, nil)
+			Expect(err).NotTo(HaveOccurred())
+
+			stopChan := make(chan struct{})
+			f, err := factory.NewWatchFactory(fakeClient, stopChan)
+			Expect(err).NotTo(HaveOccurred())
+			defer close(stopChan)
+
+			cluster := NewClusterController(fakeClient, f)
+			Expect(cluster).NotTo(BeNil())
+
+			Expect(config.OvnNorth.Address).To(Equal("watch-endpoint"), "config.OvnNorth.Address does not equal cli arg")
+			Expect(config.OvnSouth.Address).To(Equal("watch-endpoint"), "config.OvnSouth.Address does not equal cli arg")
+			Expect(config.OvnNorth.Scheme).To(Equal(config.OvnDBSchemeTCP))
+			Expect(config.OvnSouth.Scheme).To(Equal(config.OvnDBSchemeTCP))
+
+			err = cluster.watchConfigEndpoints(make(chan bool, 1))
+			Expect(err).NotTo(HaveOccurred())
+
+			// Kubernetes endpoints should eventually propogate to OvnNorth/OvnSouth
+			Eventually(func() string {
+				return config.OvnNorth.Address
+			}).Should(Equal(fmt.Sprintf("tcp:%s,tcp:%s", util.JoinHostPortInt32(masterAddress1, nbPort), util.JoinHostPortInt32(masterAddress2, nbPort))), "Northbound DB Port did not get set by watchConfigEndpoints")
+			Eventually(func() string {
+				return config.OvnSouth.Address
+			}).Should(Equal(fmt.Sprintf("tcp:%s,tcp:%s", util.JoinHostPortInt32(masterAddress1, sbPort), util.JoinHostPortInt32(masterAddress2, sbPort))), "Southbound DBPort did not get set by watchConfigEndpoints")
+
+			return nil
+		}
+		err := app.Run([]string{app.Name, "-nb-address=watch-endpoint", "-sb-address=watch-endpoint", "-manage-db-servers"})
+		Expect(err).NotTo(HaveOccurred())
+
+	})
 })
